import greenfoot.*;
import java.awt.Color;
import java.util.List;
import javax.swing.JOptionPane;

/**
 * A frog that can hatch from an egg, breed with other frogs, and even ribbit!
 * 
 * @author trimil
 */
public class Frog extends Actor
{
    private boolean egg;
    private SimpleTimer timer;
    private static final int MINUTE = 60000;
    private boolean hatchable = true;
    private boolean faceLeft;
    private long cycles;
    private int r, g, b;

    public boolean mousePressed;

    /**
     * Constructs a new frog with the specified color, direction, and egg state
     */
    public Frog(int r, int g, int b, boolean egg, boolean left)
    {
        timer = new SimpleTimer();
        this.egg = egg;
        faceLeft = left;
        this.r = r; this.g = g; this.b = b;
        if(egg)
        {
            setImage("Egg.png");
        }
        else
        {
            setImage("frog.png");
            transform(r, g, b);
        }

        if(left && !egg)
        {
            getImage().mirrorHorizontally();
        }

        cycles = 0;
        mousePressed = false;

        timer.mark();
    }

    /**
     * Constructs a new frog with the specified color, direction, and egg state
     */
    public Frog(Color col, boolean egg, boolean left)
    {
        this(col.getRed(), col.getGreen(), col.getBlue(), egg, left);
    }

    public void addedToWorld(World w)
    {
        Froggyland fl = (Froggyland) w;
        int i = 0;
        while(i < 200 && (isTouching(Trash.class) || (fl.getNurseryWidth() >= getX() && fl.getNurseryHeight() >= getY())))
        {
            setLocation(
                Greenfoot.getRandomNumber(getWorld().getWidth()), 
                Greenfoot.getRandomNumber(getWorld().getHeight()));
            i++;
        }
    }

    /**
     * Converts the frog to grayscale, then colors it the specified color.
     */
    private void transform(int r, int g, int b)
    {
        GreenfootImage img = getImage();
        for(int x = 0; x < img.getWidth(); x++)
        {
            for(int y = 0; y < img.getHeight(); y++)
            {
                img.setColorAt(x, y, grayscale(img.getColorAt(x, y)));
            }
        }
        Color newCol = new Color(r, g, b);

        int red = 0;
        int green = 0;
        int blue = 0;

        // not mine!
        for (int x = 0; x < img.getWidth(); x++) {
            for (int y = 0; y < img.getHeight(); y++) {
                Color pixel = img.getColorAt(x, y);
                int alpha = pixel.getAlpha();
                if (alpha != 0) {
                    double intensity = pixel.getGreen() / 128.0;  // mid value == 1, less is darker, more is brighter
                    red = (int)(newCol.getRed() * intensity);
                    if (red > 255) red = 255;
                    green = (int)(newCol.getGreen() * intensity);
                    if (green > 255) green = 255;
                    blue = (int)(newCol.getBlue() * intensity);
                    if (blue > 255) blue = 255;
                }
                img.setColorAt (x, y, new Color(red, green, blue, alpha));
            }
        }
    }

    /**
     * Returns the grayscale of a Color, used in {@code transform()}.
     */
    protected static Color grayscale(Color c)
    {
        int ave = (c.getRed() + c.getGreen() + c.getBlue()) / 3;
        return new Color(ave, ave, ave, c.getAlpha());
    }

    /**
     * This method does the action for the Frog.
     * the action includes:
     * <li>
     *   check if it was dragged
     *   </li><li>
     *   reporduce
     *   </li><li>
     *   ribbit
     *   </li><li>
     *   display color
     *   </li><li>
     *   hatch
     *   </li><li>
     *   and die
     * </li>
     */
    public void act() 
    {
        monitorClicks();
        
        MouseInfo mouse = Greenfoot.getMouseInfo();

        respondMouse(mouse);

        if(!egg)
        {
            Frog frog = (Frog)getOneIntersectingObject(Frog.class);
            if(frog != null && Greenfoot.mouseClicked(this) && !frog.egg && mouse.getClickCount() == 2)
            {
                reporduce(frog);
            }
        }

        if(egg && Greenfoot.mouseClicked(this) && mouse.getClickCount() == 2 && hatchable)
        {
            Froggyland land = (Froggyland)getWorld();
            if(((getX() > land.getNurseryWidth()) || (getY() > land.getNurseryHeight())) && !isTouching(Frog.class))
            {
                hatch();
            }
        }

        checkDie();

        cycles++;
    }

    private void monitorClicks()
    {
        if(Greenfoot.mousePressed(this))
        {
            mousePressed = true;
        }

        if(Greenfoot.mouseClicked(this))
        {
            mousePressed = false;
        }

    }

    private void hatch()
    {
        egg = false;
        setImage(new GreenfootImage("frog.png"));
        transform(r, g, b);
        if(faceLeft)
        {
            getImage().mirrorHorizontally();
        }
        timer.mark();
        Greenfoot.playSound("Ribbit.wav");
    }

    private void respondMouse(MouseInfo mouse)
    {
        if(Greenfoot.mouseDragged(this))
        {
            setLocation(mouse.getX(), mouse.getY());
        }

        if(!egg && Greenfoot.mouseClicked(this) && mouse.getClickCount() == 1)
        {
            Greenfoot.playSound("Ribbit.wav");
        }

        if(!egg && Greenfoot.mouseClicked(this) && mouse.getButton() == 3)
        {
            JOptionPane.showMessageDialog(null, "This frog's color is:\nColor[R = " + r + ", G = " + g + ", B = " + b + "]");
        }
    }

    /**
     * Checks if this frog should die
     */
    private void checkDie()
    {
        if(!egg && timer.millisElapsed() > 7.5*MINUTE)
        {
            die();
        }

        if(egg && timer.millisElapsed() > 20*MINUTE)
        {
            hatchable = false;
            die();
        }
    }

    /**
     * adds a new egg to the world whith the color being the average of the parent frogs, plus a random offset
     */
    private void reporduce(Frog frog)
    {
        boolean left;
        boolean m = Greenfoot.getRandomNumber(100) < 4;
        if(faceLeft && frog.faceLeft && !m)
            left = true;
        else if(!faceLeft && !frog.faceLeft && !m)
            left = false;
        else
            left = Froggyland.randBool();

        Froggyland land = (Froggyland) getWorld();
        Frog f = new Frog(getNewColor(frog.getCol()), true, left);
        int x = land.getNurseryWidth() / 2;
        int y = land.getNurseryHeight() / 2;
        x = mutate(x, -40, 40);
        y = mutate(y, -40, 40);
        getWorld().addObject(f, x, y);
    }

    /**
     * Slowly makes the frog or egg image fade away
     */
    private void die()
    {
        if((cycles % 80) == 0)
        {
            if(getImage().getTransparency() <= 1)
            {
                getWorld().removeObject(this);
                return;
            }
            getImage().setTransparency(getImage().getTransparency() - 1);
        }
    }

    /**
     * Gets the color of the frog;
     * @return The color of the frog
     */
    public Color getCol()
    {
        return new Color(r, g, b);
    }

    /**
     * Returns a color representing the average of the color of this frog and another color, plus a random offset
     * @return The average color, plus a random offset.
     */
    protected Color getNewColor(Color c)
    {
        int newR = (r + c.getRed()) / 2;
        int newG = (g + c.getGreen()) / 2;
        int newB = (b + c.getBlue()) / 2;

        newR = mutate(newR, -24, 24);
        newG = mutate(newG, -24, 24);
        newB = mutate(newB, -24, 24);

        if(newR > 255) newR = 255;
        if(newG > 255) newG = 255;
        if(newB > 255) newB = 255;

        if(newR < 0) newR = 0;
        if(newG < 0) newG = 0;
        if(newB < 0) newB = 0;

        return new Color(newR, newG, newB);
    }

    /**
     * mutates a number, that is adds a random number to it in the range min to max
     * @param
     *  num the number to mutate
     *  min the minimum mutation
     *  max the maximum mutation
     * @return num plus a random number between min and max
     */
    protected static int mutate(int num, int min, int max)
    {
        if(max < min)
        {
            return num;
        }
        int range = max - min;
        int off = Greenfoot.getRandomNumber(range) + min;
        return off + num;
    }
}
