import greenfoot.*;
import java.awt.Color;
import javax.swing.JOptionPane;

/**
 * A land of frogs
 * 
 * @author trimil
 */
public class Froggyland extends World
{
    private final int nWidth = 120;
    private final int nHeight = 140;
    private SimpleTimer customFrog;
    private static final int MINUTE = 60000;

    /**
     * Constructs a new world for frogs
     * 
     */
    public Froggyland()
    {    
        // Create a new world with 600x400 cells with a cell size of 1x1 pixels.
        super(1050, 640, 1); 
        addObject(new Trash(), 985, 585);
        frog();
        GreenfootImage img = new GreenfootImage("sand2.jpg");
        img.scale(nWidth, nHeight);
        GreenfootImage back = getBackground();
        back.drawImage(img, 0, 0);
        back.setColor(Color.BLACK);
        back.drawLine(nWidth, 0, nWidth, nHeight);
        back.drawLine(0, nHeight, nWidth, nHeight);
        setPaintOrder(Frog.class, Trash.class);
        customFrog = new SimpleTimer();
        customFrog.mark();
    }

    /**
     * Adds all the frogs to the world
     */
    private void frog()
    {
        for(int i = 0; i < 4; i++)
        {
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            if(x < nWidth && y < nHeight)
            {
                x = nWidth + 10;
                y = nHeight + 10;
            }
            int a = Greenfoot.getRandomNumber(150) - 100;
            int b = Greenfoot.getRandomNumber(200) - 100;
            int c = Greenfoot.getRandomNumber(200) - 100;
            int d = Greenfoot.getRandomNumber(200) - 100;
            addObject(new Frog(randColor(255, 0, 192, 64), true, randBool()), x + a, y - b);
            addObject(new Frog(randColor(255, 0, 192, 64), true, randBool()), x - b, y + d);
            addObject(new Frog(randColor(255, 0, 192, 64), true, randBool()), x + c, y + a);
            addObject(new Frog(randColor(255, 0, 192, 64), true, randBool()), x - d, y - c);
        }
    }

    /**
     * Returns a random boolean value, used in the creation of frogs
     * @return a random boolean value
     */
    public static boolean randBool()
    {
        return Greenfoot.getRandomNumber(1000) < 450;
    }

    /**
     * Returns a random {@code color}
     * @param
     * 		max the maximum value for the {@code color} components
     * 		min the minimum value for the {@code color} components
     * 		gmax a value that determens the brightness of the {@code color}
     * 		gmin a value that determens the brightness of the {@code color}
     * @return a random {@code color} object
     */
    private Color randColor(int max, int min, int gmax, int gmin)
    {
        int r = Greenfoot.getRandomNumber(max - min) + min;
        int g = Greenfoot.getRandomNumber(max - min) + min;
        int b = Greenfoot.getRandomNumber(max - min) + min;
        if(r < gmin && (g < gmax || b < gmax))
        {
            r = gmin;
        }
        if(g < gmin && (r < gmax || b < gmax))
        {
            g = gmin;
        }
        if(b < gmin && (g < gmax || r < gmax))
        {
            b = gmin;
        }
        return new Color(r, g, b);
    }

    /**
     * Returns the width of the nursery area
     * @return the width of the nursery
     */
    public int getNurseryWidth()
    {
        return nWidth;
    }

    /**
     * Returns the height of the nursery area
     * @return the width of the nursery
     */
    public int getNurseryHeight()
    {
        return nHeight;
    }
    
    /**
     * Checks for mouse clicks and checks frogs every {@code act()} cycle
     */
    @Override
    public void act()
    {
        if(Greenfoot.mouseClicked(this))
        {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            if(mouse.getX() < nWidth && mouse.getY() < nHeight && customFrog.millisElapsed() > 1.5*MINUTE)
            {
                addCustom();
            }
        }
        if(getObjects(Frog.class).isEmpty())
        {
            Greenfoot.stop();
            JOptionPane.showMessageDialog(null, "Oops, all frogs died!", "Game over", JOptionPane.INFORMATION_MESSAGE);
        }
    }

    /**
     * Adds a frog with a customized R, G, and B components and direction
     */
    protected void addCustom()
    {
        String title = "Custom Frog";
        String[] comps = {"red", "green", "blue"};
        int want = JOptionPane.showConfirmDialog(null, "Do you want to create a custom frog?", title, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null);
        if(want == JOptionPane.YES_OPTION)
        {
            int[] ans = {0, 0, 0};
            for(int i = 0; i < 3; i++)
            {
                String str = JOptionPane.showInputDialog(null, "Enter " + comps[i] + " value:", "");
                if(str == null)
                    return;
                ans[i] = new Integer(str);
            }
            int faceLeft = JOptionPane.showConfirmDialog(null, "Would you like it to face left?", title, JOptionPane.YES_NO_OPTION);
            boolean left = (faceLeft == JOptionPane.YES_OPTION);
            for(int i = 0; i < 3; i++)
            {
                if(ans[i] < 45)
                    ans[i] = 45;
            }
            Frog f = new Frog(ans[0], ans[1], ans[2], true, left);
            addObject(f, Greenfoot.getRandomNumber(nWidth), Greenfoot.getRandomNumber(nHeight));
            customFrog.mark();
        }
    }
}
